@@ -98,6 +98,14 @@ let string_fold_right f s acc =
     else loop (i-1) (f s.[i] acc) in
   loop (len-1) acc
 
+let cut_exp_zero s =
+  match String.split_on_char 'e' s with
+  | [signif;exponent] ->
+    (match exponent.[0] with (* int_of_string removes a leading '0' *)
+     | '+' -> Printf.sprintf "%se+%i" signif (int_of_string exponent) (* keep a leading '+' *)
+     | _   -> Printf.sprintf "%se%i" signif (int_of_string exponent)) (* keep a leading '-' *)
+  | _ -> s
+
 exception No_example_found of string
 (* raised if an example failed to be found *)
 
@@ -165,9 +173,16 @@ module Gen = struct
 
   let bool st = RS.bool st
 
-  let float st =
-    exp (RS.float st 15. *. (if RS.float st 1. < 0.5 then 1. else -1.))
-    *. (if RS.float st 1. < 0.5 then 1. else -1.)
+  let float st = (* switch to [bits64] once lower bound reaches 4.14 *)
+    (* Technically we could write [15] but this is clearer *)
+    let four_bits_mask = 0b1111 in
+    (* Top 4 bits *)
+    let left = Int64.(shift_left (of_int (RS.bits st land four_bits_mask)) 60) in
+    (* Middle 30 bits *)
+    let middle = Int64.(shift_left (of_int (RS.bits st)) 30) in
+    (* Bottom 30 bits *)
+    let right = Int64.of_int (RS.bits st) in
+    Int64.(float_of_bits (logor left (logor middle right)))
 
   let pfloat st = abs_float (float st)
   let nfloat st = -.(pfloat st)
@@ -495,7 +510,12 @@ module Print = struct
   let int32 i = Int32.to_string i ^ "l"
   let int64 i = Int64.to_string i ^ "L"
   let bool = string_of_bool
-  let float = string_of_float
+  let float f = (* Workaround for Windows and macOS to print negative nans consistently as "-nan" *)
+    if Float.is_nan f && Float.sign_bit f
+    then "-nan"
+    else if Sys.win32 (* Windows workaround to avoid leading exponent zero such as "-1.00001604579e-010" *)
+    then string_of_float f |> cut_exp_zero
+    else string_of_float f
   let string s = Printf.sprintf "%S" s
   let bytes b = string (Bytes.to_string b)
   let char c = Printf.sprintf "%C" c
@@ -736,6 +756,93 @@ module Shrink = struct
     if x>0 then for i=x-1 downto 0 do yield i done;
     if x<0 then for i=x+1 to 0 do yield i done
 
+  let float_suff_different cand x =
+    let threshold = 1e-4 in (* candidate has to be at least 0.0001% different *)
+    Float.(abs cand < abs x) && Float.abs ((cand -. x) /. x) > threshold
+
+  (* [float_shrink_exponent 2.234 213] shrinks an exponent 213 from 2.234e213
+     and glues it back together with significand 2.234 *)
+  let float_shrink_exponent signif exponent yield =
+    int (int_of_string exponent)
+      (fun exponent -> Printf.sprintf "%se%i" signif exponent |> float_of_string |> yield)
+
+  (* [float_shrink_significand 2.234e213 "2.234" "213"] shrinks the significand 2.234 of
+     a floating point number in scientific notation 2.234e213 *)
+  let float_shrink_significand orig_x signif exponent yield =
+    let signif = float_of_string signif in
+    let exponent = float_of_string ("1e" ^ exponent) in
+    let recompose_float signif = signif *. exponent in
+    let recompose_and_yield signif =
+      let cand = recompose_float signif in
+      if String.length (Print.float cand) <= String.length (Print.float orig_x)
+      then yield cand in
+    (* [shrink_decimals 2.2345 1000] multiplies a significand 2.2345 with a
+       precision 1000 to get [2234], shrink it, and thus round off decimals *)
+    let shrink_decimals signif prec yield =
+      let tmp = signif *. float prec in (* use tmp to avoid i386 ocamlopt 4 weirdness https://github.com/ocaml/ocaml/issues/8018 *)
+      let multiple = int_of_float tmp in (* returns an unshrinkable 0 on overflow *)
+      if prec = 10 || multiple mod 100 <> 0 then (* first iteration or decimals to round *)
+        int multiple (fun i ->
+            let signif' = float i /. float prec in
+            if float_suff_different signif' signif
+            then yield signif') in
+    (* first try reducing the decimal digits with different precision *)
+    if signif > 1.0 || signif < -1.0 (* don't attempt to shrink 1.0 or -1.0 *)
+    then
+      begin
+        shrink_decimals signif 10     recompose_and_yield;
+        shrink_decimals signif 1000   recompose_and_yield;
+        shrink_decimals signif 100000 recompose_and_yield
+      end;
+    (* second try simple reductions of the significand's leading digit, in [1;9] *)
+    if signif > 2. then
+      (* shrink  2.234e213 to  1.234e213 by int shrinking the leading 2 *)
+      int (int_of_float signif)
+        (fun s -> if s != 0 then yield (recompose_float (signif -. floor signif +. float s)));
+    if signif < -2. then
+      (* shrink -2.234e213 to -1.234e213 by int shrinking the leading -2 *)
+      int (int_of_float signif)
+        (fun s -> if s != 0 then yield (recompose_float (signif -. ceil signif +. float s)))
+
+  let float x yield =
+    if not (Float.is_infinite x || Float.is_nan x) then
+    (* first try quick roundings and negation *)
+    (if x > 1. then let floor_x = floor x in
+       if float_suff_different floor_x x then yield floor_x);
+    (if x < -1. then let ceil_x = ceil x in
+       if float_suff_different ceil_x x then yield ceil_x);
+    if x < 0. then yield (-. x);
+    (* second proceed with simplification based on decimal, scientific notation 2.234e213 *)
+    match String.split_on_char 'e' (Printf.sprintf "%e" x) with
+    | [signif;exponent] ->
+      float_shrink_exponent signif exponent yield;
+      float_shrink_significand x signif exponent yield
+    | _ -> ()
+
+  let float_bound bound =
+    if bound > 0.
+    then
+      fun i ->
+        Iter.map (fun i -> i -. 1.) (float (i +. 1.)) (* towards 1. and subtract 1. afterwards *)
+        |> Iter.filter (fun i -> 0. <= i && i < bound)
+    else
+      fun i ->
+        Iter.map (fun i -> i +. 1.) (float (i -. 1.)) (* towards -1. and add 1. afterwards *)
+        |> Iter.filter (fun i -> bound < i && i <= 0.)
+
+  let float_range low high =
+    if high < low then invalid_arg "Shrink.float_range: invalid range";
+    let filter_range = Iter.filter (fun i -> low <= i && i <= high) in
+    if low >= 0. then
+      fun i -> (* move [low;high] to [1;high-low+1], shrink towards 1., and move back *)
+        Iter.map (fun i -> i +. low -. 1.) (float (i -. low +. 1.)) |> filter_range
+    else if high <= 0. then
+      fun i -> (* move [low;high] to [low-high-1;-1], shrink towards -1., and move back *)
+        Iter.map (fun i -> i +. high +. 1.) (float (i -. high -. 1.)) |> filter_range
+    else (* low < 0 && 0 < high, i.e., crosses 0. *)
+      fun i -> (* shrink towards 1. and subtract 1. afterwards *)
+        Iter.map (fun i -> i -. 1.) (float (i +. 1.)) |> filter_range
+
   let filter f shrink x = Iter.filter f (shrink x)
 
   let char_generic target c =
@@ -1115,8 +1222,8 @@ let get_print o = o.print
 
 let small1 _ = 1
 
-let make_scalar ?print ?collect gen =
-  make ~shrink:Shrink.nil ~small:small1 ?print ?collect gen
+let make_scalar ?collect gen =
+  make ~shrink:Shrink.float ~small:small1 ~print:Print.float ?collect gen
 let make_int ?collect gen =
   make ~shrink:Shrink.int ~small:small1 ~print:Print.int ?collect gen
 
@@ -1136,19 +1243,21 @@ let unit : unit arbitrary =
   make ~small:small1 ~shrink:Shrink.nil ~print:Print.unit Gen.unit
 let bool =
   make ~small:small1 ~shrink:Shrink.bool ~print:Print.bool Gen.bool
-let float = make_scalar ~print:Print.float Gen.float
-let pos_float = make_scalar ~print:Print.float Gen.pfloat
-let neg_float = make_scalar ~print:Print.float Gen.nfloat
+
+let float = make_scalar Gen.float
+let pos_float = make_scalar Gen.pfloat
+let neg_float = make_scalar Gen.nfloat
 
 let float_bound_inclusive bound =
-  make_scalar ~print:Print.float (Gen.float_bound_inclusive bound)
+  make ~small:small1 ~shrink:(Shrink.float_bound bound) ~print:Print.float (Gen.float_bound_inclusive bound)
 
 let float_bound_exclusive bound =
-  make_scalar ~print:Print.float (Gen.float_bound_exclusive bound)
+  make ~small:small1 ~shrink:(Shrink.float_bound bound) ~print:Print.float (Gen.float_bound_exclusive bound)
 
-let float_range low high = make_scalar ~print:Print.float (Gen.float_range low high)
+let float_range low high =
+  make ~small:small1 ~shrink:(Shrink.float_range low high) ~print:Print.float (Gen.float_range low high)
 
-let exponential mean = make_scalar ~print:Print.float (Gen.exponential mean)
+let exponential mean = make_scalar (Gen.exponential mean)
 
 let int = make_int Gen.int
 let int_bound n = make_int (Gen.int_bound n)
@@ -1245,7 +1354,7 @@ let small_list a = mk_list a (Gen.small_list a.gen)
 let array_sum_ f a = Array.fold_left (fun acc x -> f x+acc) 0 a
 
 let array a =
-  let small = _opt_map_or ~d:Array.length ~f:array_sum_ a.small in
+  let small x = _opt_map_or ~d:Array.length ~f:array_sum_ a.small x in
   make
     ~small
     ~shrink:(Shrink.array ?shrink:a.shrink)
@@ -1253,7 +1362,7 @@ let array a =
     (Gen.array a.gen)
 
 let array_of_size size a =
-  let small = _opt_map_or ~d:Array.length ~f:array_sum_ a.small in
+  let small x = _opt_map_or ~d:Array.length ~f:array_sum_ a.small x in
   make
     ~small
     ~shrink:(Shrink.array ?shrink:a.shrink)
