@@ -776,6 +776,13 @@ end
 
     Shrinking is defined as a type {!Shrink.t} that takes an argument to shrink
     and produces an iterator of type {!Iter.t} of shrinking candidates.
+
+    ⚠️ In order to function well, a shrinker must:
+    - never return its argument as a shrinking candidate, and
+    - return shrinking candidates which are smaller by some termination measure
+      ([list] length, tree depth, number of bits, ...)
+
+    Failure to do so, may result in an infinite shrinker loop.
 *)
 
 (** {2 Iterators} *)
@@ -872,8 +879,11 @@ module Shrink : sig
   (** The [Shrink] module contains combinators to build up composite shrinkers
       for user-defined types
 
-      Warning: A hand-written shrinker returning its own argument, will cause
-      QCheck's shrinking phase to loop infinitely. *)
+      ⚠️ Warning: [QCheck]'s shrinking phase may loop infinitely if
+      - the shrinker returns its own argument as a shrinking candidate, or
+      - fails to return a strictly smaller shrinking candidate by some termination measure
+        (e.g., a [list] permutation may lead to an infinite shrinking cycle).
+  *)
 
   type 'a t = 'a -> 'a Iter.t
   (** Given a counter-example, return an iterator on smaller versions
@@ -913,6 +923,24 @@ module Shrink : sig
   (** int64 shrinker. Shrinks towards [0L].
       @since 0.14 *)
 
+  val float : float t
+  (** float shrinker. Shrinks floating point numbers towards [1.0] or [-1.0]
+      and prefers a shorter printed rendering.
+      @since 0.27 *)
+
+  val float_bound : float -> float t
+  (** float shrinker for bounded floating point numbers.
+      [float_bound b] shrinks floating point numbers towards [0.] for both
+      positive and negative bounds [b], preferring a shorter printed rendering.
+      @since 0.27 *)
+
+  val float_range : float -> float -> float t
+  (** [float_range low high] shrinks floating point numbers in the range [low; high] (inclusive).
+      Shrinks towards [low] if [low >= 0.], towards [high] if [high <= 0.], and
+      towards [0.] if [low < 0.] and [0. <. high]. It prefers a shorter printed rendering.
+      @raise Invalid_argument if [low > high].
+      @since 0.27 *)
+
   val option : 'a t -> 'a option t
   (** option shrinker. Shrinks towards [None].
       [option shk] reduces [Some v] values using [shk] to reduce [v]. *)
@@ -1094,14 +1122,13 @@ val bool : bool arbitrary
 (** Uniform boolean generator. *)
 
 val float : float arbitrary
-(** Generates regular floats (no nan and no infinities). *)
-(* FIXME: does not generate nan nor infinity I think. *)
+(** Generates regular floats (no infinities). *)
 
 val pos_float : float arbitrary
-(** Positive float generator (no nan and no infinities). *)
+(** Positive float generator (no infinities). *)
 
 val neg_float : float arbitrary
-(** Negative float generator (no nan and no infinities). *)
+(** Negative float generator (no infinities). *)
 
 val float_bound_inclusive : float -> float arbitrary
 (** [float_bound_inclusive n] is uniform between [0] and [n] included. If [bound] is
