@@ -340,6 +340,12 @@ module Generator = struct
       IntTree.gen_tree
       (fun tree -> IntTree.(rev_tree (rev_tree tree)) = tree)
 
+  let float_test =
+    Test.make ~name:"regression test negative float range" ~count:1000
+      ~print:Print.float
+      (Gen.float_range (-2.) (-1.))
+      (fun _ -> false)
+
   let tests = [
     char_dist_issue_23;
     char_test;
@@ -368,6 +374,7 @@ module Generator = struct
     int_option_test;
     int_string_result_test;
     passing_tree_rev;
+    float_test;
   ]
 end
 
@@ -432,6 +439,34 @@ module Shrink = struct
     Test.make ~name:"nat < 5001" ~count:1000 ~print:Print.int
       Gen.nat (fun n -> n < 5001)
 
+  let float_leq_1e10 =
+    Test.make ~name:"float <= 1e10" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f <= 1e10)
+
+  let float_lt_pi =
+    Test.make ~name:"float < Float.pi" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f < Float.pi)
+
+  let float_leq_1 =
+    Test.make ~name:"float <= 1.0" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f <= 1.0)
+
+  let float_lt_1 =
+    Test.make ~name:"float < 1.0" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f < 1.0)
+
+  let float_leq_1em10 =
+    Test.make ~name:"float <= 1e-10" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f <= 1e-10)
+
+  let float_geq_m1em10 =
+    Test.make ~name:"float >= -1e-10" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f >= -1e-10)
+
+  let float_geq_m1e10 =
+    Test.make ~name:"float >= -1e10" ~count:1000 ~print:Print.float
+      Gen.float (fun f -> f >= -1e10)
+
   let char_is_never_abcdef =
     Test.make ~name:"char never produces 'abcdef'" ~count:1000 ~print:Print.char
       Gen.char (fun c -> not (List.mem c ['a';'b';'c';'d';'e';'f']))
@@ -688,6 +723,7 @@ module Shrink = struct
       Gen.(no_shrink @@ list small_int)
       (fun xs -> List.fold_left (+) 0 xs = 0)
 
+
   let tests = [
     (*test_fac_issue59;*)
     big_bound_issue59;
@@ -700,6 +736,13 @@ module Shrink = struct
     int64s_are_0L;
     ints_smaller_209609;
     nats_smaller_5001;
+    float_leq_1e10;
+    float_lt_pi;
+    float_leq_1;
+    float_lt_1;
+    float_leq_1em10;
+    float_geq_m1em10;
+    float_geq_m1e10;
     char_is_never_abcdef;
     printable_is_never_sign;
     numeral_is_never_less_5;
@@ -944,11 +987,11 @@ module Stats = struct
 
   let option_dist =
     Test.make ~name:"option dist" ~count:10_000
-      ~collect:(function None -> "None  " | Some _ -> "Some _") Gen.(option int) (fun _ -> true)
+      ~collect:(function None -> "None" | Some _ -> "Some _") Gen.(option int) (fun _ -> true)
 
   let result_dist =
     Test.make ~name:"result dist" ~count:10_000
-      ~collect:(function Ok _ -> "Ok _   " | Error _ -> "Error _") Gen.(result int string) (fun _ -> true)
+      ~collect:(function Ok _ -> "Ok _" | Error _ -> "Error _") Gen.(result int string) (fun _ -> true)
 
   let list_len_tests =
     let len = ("len",List.length) in
@@ -1003,6 +1046,21 @@ module Stats = struct
       Test.make ~name:"exponential -10. dist" ~count:5_000 ~stats:[float_dist] (Gen.exponential (-10.)) (fun _ -> true);
     ]
 
+  let float_tests = (* Float.frexp nan (and infinity) is undefined and may return a 32766 exponent on Alpine *)
+    let float_expon_dist  = ("exponent",    fun f -> if Float.(is_nan f || is_infinite f) then 0 else snd (Float.frexp f)) in
+    let float_signif_dist = ("significant", fun f -> if Float.(is_nan f || is_infinite f) (* int_of_float is undefined on nan and infinity *)
+                                                     then 0 else let s = fst (Float.frexp f) in int_of_float (s *. 1000000.)) in
+    let float_fpclass f = match Float.classify_float f with
+      | FP_normal    -> "FP_normal"
+      | FP_subnormal -> "FP_subnormal"
+      | FP_zero      -> "FP_zero"
+      | FP_infinite  -> "FP_infinite"
+      | FP_nan       -> "FP_nan" in
+    [ Test.make ~name:"float exponent"    ~count:5000 ~stats:[float_expon_dist] Gen.float (fun _ -> true);
+      Test.make ~name:"float significant" ~count:5000 ~stats:[float_signif_dist] Gen.float (fun _ -> true);
+      Test.make ~name:"float classify"    ~count:5000 ~collect:float_fpclass Gen.float (fun _ -> true);
+    ]
+
   let tree_depth_test =
     let depth = ("depth", IntTree.depth) in
     Test.make ~name:"tree's depth" ~count:1000 ~stats:[depth] IntTree.gen_tree (fun _ -> true)
@@ -1028,4 +1086,5 @@ module Stats = struct
     @ int_dist_tests
     @ int_32_64_dist_tests
     @ exponential_tests
+    @ float_tests
 end
